@@ -1,7 +1,7 @@
 ## NOTE: this has problems when '...' contains quoted names that have special symbols
 ## for a better option see the logic in setkey
 
-getdots <- function()
+getdots = function()
 {
   # return a string vector of the arguments in '...'
   # My long winded way: gsub(" ","",unlist(strsplit(deparse(substitute(list(...))),"[(,)]")))[-1]
