@@ -1,7 +1,10 @@
 #ifndef dt_FREAD_R_H
 #define dt_FREAD_R_H
-#define STRICT_R_HEADERS   // https://cran.r-project.org/doc/manuals/r-devel/R-exts.html#Error-handling
+#ifndef STRICT_R_HEADERS
+  #define STRICT_R_HEADERS   // https://cran.r-project.org/doc/manuals/r-devel/R-exts.html#Error-handling
+#endif
 #include <R.h>
+#include <Rinternals.h>
 #include "po.h"
 
 #define FREAD_MAIN_ARGS_EXTRA_FIELDS \
@@ -18,6 +21,8 @@
 // Where no halt is happening, we can just use raw Rprintf() or warning()
 void __halt(bool warn, const char *format, ...);   // see freadR.c
 #define STOP(...)   __halt(0, __VA_ARGS__)
+static char internal_error_buff[1001] __attribute__((unused)); // match internalErrSize // todo: fix imports such that compiler warns correctly #6468
+#define INTERNAL_STOP(...) do {snprintf(internal_error_buff, 1000, __VA_ARGS__); __halt(0, "%s %s: %s. %s", _("Internal error in"), __func__, internal_error_buff, _("Please report to the data.table issues tracker"));} while (0)
 #define DTPRINT     Rprintf
 #define DTWARN(...) warningsAreErrors ? __halt(1, __VA_ARGS__) : warning(__VA_ARGS__)
 
